import pymatbridge as pymat
import IPython

import numpy.testing as npt

class TestMagic:

    # Create an IPython shell and load Matlab magic
    @classmethod
    def setup_class(cls):
        cls.ip = IPython.InteractiveShell()
        cls.ip.run_cell('import random')
        cls.ip.run_cell('import numpy as np')
        pymat.load_ipython_extension(cls.ip)

    # Unload the magic, shut down Matlab
    @classmethod
    def teardown_class(cls):
        pymat.unload_ipython_extension(cls.ip)


    # Test single operation on different data structures
    def test_cell_magic_number(self):
        # A double precision real number
        self.ip.run_cell("a = np.float64(random.random())")
        self.ip.run_cell_magic('matlab', '-i a -o b', 'b = a*2;')
        npt.assert_almost_equal(self.ip.user_ns['b'],
                                self.ip.user_ns['a']*2, decimal=7)

    def test_cell_magic_number_complex(self):
        # A complex number
        self.ip.run_cell("x = 3.34+4.56j")
        self.ip.run_cell_magic('matlab', '-i x -o y', 'y = x*(11.35 - 23.098j)')
        self.ip.run_cell("res = x*(11.35 - 23.098j)")
        npt.assert_almost_equal(self.ip.user_ns['y'],
                                self.ip.user_ns['res'], decimal=7)


    def test_cell_magic_array(self):
        # Random array multiplication
        self.ip.run_cell("val1 = np.random.random_sample((3,3))")
        self.ip.run_cell("val2 = np.random.random_sample((3,3))")
        self.ip.run_cell("respy = np.dot(val1, val2)")
        self.ip.run_cell_magic('matlab', '-i val1,val2 -o resmat',
                               'resmat = val1 * val2')
        npt.assert_almost_equal(self.ip.user_ns['resmat'],
                                self.ip.user_ns['respy'], decimal=7)

    def test_cell_magic_array_complex(self):
        self.ip.run_cell("val1 = np.random.random((3,3)) + np.random.random((3,3))*1j")
        self.ip.run_cell("val2 = np.random.random((3,3)) + np.random.random((3,3))*1j")
        self.ip.run_cell("respy = np.dot(val1, val2)")
        self.ip.run_cell_magic('matlab', '-i val1,val2 -o resmat',
                               'resmat = val1 * val2')
        npt.assert_almost_equal(self.ip.user_ns['resmat'],
                                self.ip.user_ns['respy'], decimal=7)

    def test_line_magic(self):
        # Some operation in Matlab
        self.ip.run_line_magic('matlab', 'a = [1 2 3]')
        self.ip.run_line_magic('matlab', 'res = a*2')
        # Get the result back to Python
        self.ip.run_cell_magic('matlab', '-o actual', 'actual = res')

        self.ip.run_cell("expected = np.array([[2., 4., 6.]])")
        npt.assert_almost_equal(self.ip.user_ns['actual'],
                                self.ip.user_ns['expected'], decimal=7)

    def test_figure(self):
        # Just make a plot to get more testing coverage
        self.ip.run_line_magic('matlab', 'plot([1 2 3])')


    def test_matrix(self):
        self.ip.run_cell("in_array = np.array([[1,2,3], [4,5,6]])")
        self.ip.run_cell_magic('matlab', '-i in_array -o out_array',
                               'out_array = in_array;')
        npt.assert_almost_equal(self.ip.user_ns['out_array'],
                                self.ip.user_ns['in_array'],
                                decimal=7)

    # Matlab struct type should be converted to a Python dict
    def test_struct(self):
        self.ip.run_cell('num = 2.567')
        self.ip.run_cell('num_array = np.array([1.2,3.4,5.6])')
        self.ip.run_cell('str = "Hello World"')
        self.ip.run_cell_magic('matlab', '-i num,num_array,str -o obj',
                                'obj.num = num;obj.num_array = num_array;obj.str = str;')
        npt.assert_equal(isinstance(self.ip.user_ns['obj'], dict), True)
        npt.assert_equal(self.ip.user_ns['obj']['num'], self.ip.user_ns['num'])
        npt.assert_equal(self.ip.user_ns['obj']['num_array'].squeeze(), self.ip.user_ns['num_array'])
        npt.assert_equal(self.ip.user_ns['obj']['str'], self.ip.user_ns['str'])



import pymatbridge as pymat
import numpy.testing as npt
import test_utils as tu


class TestFunctionProcessor:

    # Start a Matlab session before running any tests
    @classmethod
    def setup_class(cls):
        cls.mlab = tu.connect_to_matlab()

    # Tear down the Matlab session after running all the tests
    @classmethod
    def teardown_class(cls):
        tu.stop_matlab(cls.mlab)

    # Test the "is_function_processor_working()" function
    def test_is_function_processor_working(self):
        npt.assert_equal(self.mlab.is_function_processor_working(), True)



import pymatbridge as pymat
import numpy.testing as npt
import test_utils as tu


class TestGetVariable:

    # Start a Matlab session before running any tests
    @classmethod
    def setup_class(cls):
        cls.mlab = tu.connect_to_matlab()

    # Tear down the Matlab session after running all the tests
    @classmethod
    def teardown_class(cls):
        tu.stop_matlab(cls.mlab)


    # Set up a floating point and get it
    def test_get_float(self):
        self.mlab.run_code("a = 456345.345345")
        self.mlab.run_code("b = 0.39748e3")

        npt.assert_equal(self.mlab.get_variable('a'), 456345.345345)
        npt.assert_equal(self.mlab.get_variable('b'), 397.48)


    # Get some arrays
    def test_get_array(self):
        self.mlab.run_code("a = [1 2 3 4]")
        self.mlab.run_code("b = [1 2; 3 4]")

        npt.assert_equal(self.mlab.get_variable('a'), [[1.,2.,3.,4.]])
        npt.assert_equal(self.mlab.get_variable('b'), [[1.,2.],[3.,4.]])


    # Try to get a non-existent variable
    def test_nonexistent_var(self):
        self.mlab.run_code("clear")

        npt.assert_equal(self.mlab.get_variable('a'), None)


    # Try to get a non-existent variable with default
    def test_nonexistent_var_default(self):
        self.mlab.run_code("clear")

        npt.assert_equal(self.mlab.get_variable('a', 'some_val'), 'some_val')



import pymatbridge as pymat
import random as rd
import numpy as np
import numpy.testing as npt
import test_utils as tu


class TestPrecision:

    # Start a Matlab session before running any tests
    @classmethod
    def setup_class(cls):
        cls.mlab = tu.connect_to_matlab()

    # Tear down the Matlab session after running all the tests
    @classmethod
    def teardown_class(cls):
        tu.stop_matlab(cls.mlab)


    # Pass a 64-bit floating number through the signal path
    def test_float64_roundtrip(self):
        for i in range(0,10):
            val = np.float64(rd.random())
            res = self.mlab.run_func('precision_pass.m', {'val':val})['result']
            npt.assert_almost_equal(res, val, decimal=7, err_msg="Float64 roundtrip error")

    # Add two 64-bit floating number in Matlab and return the sum
    def test_float64_sum(self):
        for i in range(0,10):
            val1 = np.float64(rd.random())
            val2 = np.float64(rd.random())

            res = self.mlab.run_func('precision_sum.m', {'val1':val1, 'val2':val2})['result']
            npt.assert_almost_equal(res, val1 + val2, decimal=7, err_msg="Float64 sum error")

    # Multiply two 64-bit floating number in Matlab and return the product
    def test_float64_multiply(self):
        for i in range(0,10):
            val1 = np.float64(rd.random())
            val2 = np.float64(rd.random())

            res = self.mlab.run_func('precision_multiply.m', {'val1':val1, 'val2':val2})['result']
            npt.assert_almost_equal(res, val1 * val2, decimal=7, err_msg="Float64 multiply error")

    # Make a division in Matlab and return the results
    def test_float64_divide(self):
        for i in range(0,10):
            val1 = np.float64(rd.random())
            val2 = np.float64(rd.random())

            res = self.mlab.run_func('precision_divide.m', {'val1':val1, 'val2':val2})['result']
            npt.assert_almost_equal(res, val1 / val2, decimal=7, err_msg="Float64 divide error")

    # Calculate the square root in Matlab and return the result
    def test_float64_sqrt(self):
        for i in range(0,10):
            val = np.float64(rd.random())

            res = self.mlab.run_func('precision_sqrt.m', {'val':val})['result']
            npt.assert_almost_equal(res, np.sqrt(val), decimal=7, err_msg="Float64 square root error")



import os

import pymatbridge as pymat
import numpy.testing as npt

def on_octave():
    return bool(os.environ.get('USE_OCTAVE', False))

def connect_to_matlab():
    mlab = pymat.Octave() if on_octave() else pymat.Matlab()
    mlab.start()
    npt.assert_(mlab.is_connected(), msg = "connect_to_matlab(): Connection failed")

    return mlab


def stop_matlab(mlab):
    mlab.stop()
    npt.assert_(not mlab.is_connected(), msg = "stop_matlab(): Disconnection failed")




"""
pymatbridge
===========

This is a module for communicating and running Matlab from within python

Example
-------

>>> import pymatbridge
>>> m = pymatbridge.Matlab()
>>> m.start()
Starting MATLAB on ZMQ socket ipc:///tmp/pymatbridge
Send 'exit' command to kill the server
.MATLAB started and connected!
True
>>> 
                            < M A T L A B (R) >
                  Copyright 1984-2014 The MathWorks, Inc.
                   R2014b (8.4.0.150421) 64-bit (maci64)
                             September 15, 2014

 
To get started, type one of these: helpwin, helpdesk, or demo.
For product information, visit www.mathworks.com.
 
Socket created at: ipc:///tmp/pymatbridge

>>> 
>>> m.run_code('a=1;')
{'content': {'stdout': '', 'datadir': '/private/tmp/MatlabData/', 'code': 'a=1;', 'figures': []}, 'success': 'true'}
>>> m.get_variable('a')
1

"""

import os
import time
import codecs
import zmq
import subprocess
import sys
import json
from uuid import uuid4

try:
    from numpy import ndarray, generic, float64, frombuffer, asfortranarray
except ImportError:
    class ndarray:
        pass
    generic = ndarray

try:
    from scipy.sparse import spmatrix
except ImportError:
    class spmatrix:
        pass


def encode_ndarray(obj):
    """Write a numpy array and its shape to base64 buffers"""
    shape = obj.shape
    if len(shape) == 1:
        shape = (1, obj.shape[0])
    if obj.flags.c_contiguous:
        obj = obj.T
    elif not obj.flags.f_contiguous:
        obj = asfortranarray(obj)
    data = obj.astype(float64).tobytes()
    data = codecs.encode(data, 'base64').decode('utf-8')
    return data, shape


# JSON encoder extension to handle complex numbers and numpy arrays
class PymatEncoder(json.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, ndarray) and obj.dtype.kind in 'uif':
            data, shape = encode_ndarray(obj)
            return {'ndarray': True, 'shape': shape, 'data': data}
        elif isinstance(obj, ndarray) and obj.dtype.kind == 'c':
            real, shape = encode_ndarray(obj.real.copy())
            imag, _ = encode_ndarray(obj.imag.copy())
            return {'ndarray': True, 'shape': shape,
                    'real': real, 'imag': imag}
        elif isinstance(obj, ndarray):
            return obj.tolist()
        elif isinstance(obj, complex):
            return {'real': obj.real, 'imag': obj.imag}
        elif isinstance(obj, generic):
            return obj.item()
        # Handle the default case
        return json.JSONEncoder.default(self, obj)


def decode_arr(data):
    """Extract a numpy array from a base64 buffer"""
    data = data.encode('utf-8')
    return frombuffer(codecs.decode(data, 'base64'), float64)


# JSON decoder for arrays and complex numbers
def decode_pymat(dct):
    if 'ndarray' in dct and 'data' in dct:
        value = decode_arr(dct['data'])
        shape = decode_arr(dct['shape'])
        return value.reshape(shape, order='F')
    elif 'ndarray' in dct and 'imag' in dct:
        real = decode_arr(dct['real'])
        imag = decode_arr(dct['imag'])
        shape = decode_arr(dct['shape'])
        data = real + 1j * imag
        return data.reshape(shape, order='F')
    elif 'real' in dct and 'imag' in dct:
        return complex(dct['real'], dct['imag'])
    return dct

MATLAB_FOLDER = '%s/matlab' % os.path.realpath(os.path.dirname(__file__))


class _Session(object):
    """
    A class for communicating with a MATLAB session. It provides the behavior
    common across different MATLAB implementations. You shouldn't instantiate
    this directly; rather, use the Matlab or Octave subclasses.
    """

    def __init__(self, executable, socket_addr=None,
                 id='python-matlab-bridge', log=False, maxtime=60,
                 platform=None, startup_options=None):
        """
        Initialize this thing.

        Parameters
        ----------

        executable : str
            A string that would start the session at the terminal.

        socket_addr : str
            A string that represents a valid ZMQ socket address, such as
            "ipc:///tmp/pymatbridge", "tcp://127.0.0.1:55555", etc.

        id : str
            An identifier for this instance of the pymatbridge.

        log : bool
            Whether to save a log file in some known location.

        maxtime : float
           The maximal time to wait for a response from the session (optional,
           Default is 10 sec)

        platform : string
           The OS of the machine on which this is running. Per default this
           will be taken from sys.platform.

        startup_options : string
           Command line options to include in the executable's invocation.
           Optional; sensible defaults are used if this is not provided.
        """
        self.started = False
        self.running = False
        self.executable = executable
        self.socket_addr = socket_addr
        self.id = id
        self.log = log
        self.maxtime = maxtime
        self.platform = platform if platform is not None else sys.platform
        self.startup_options = startup_options

        if socket_addr is None:
            self.socket_addr = "tcp://127.0.0.1:55555" if self.platform == "win32" else "ipc:///tmp/pymatbridge"

        if self.log:
            startup_options += ' > ./pymatbridge/logs/bashlog_%s.txt' % self.id

        self.context = None
        self.socket = None

    def _program_name(self):
        raise NotImplemented

    def _preamble_code(self):
        # suppress warnings while loading the path, in the case of
        # overshadowing a built-in function on a newer version of
        # Matlab (e.g. isrow)
        return ["warning('off','all')",
                "addpath(genpath('%s'))" % MATLAB_FOLDER,
                "warning('on', 'all')"]

    def _execute_flag(self):
        raise NotImplemented

    def _run_server(self):
        code = self._preamble_code()
        code.extend([
            "matlabserver('%s')" % self.socket_addr,
            'exit'
        ])
        command = '%s %s %s "%s"' % (self.executable, self.startup_options,
                                     self._execute_flag(), ','.join(code))
        subprocess.Popen(command, shell=True, stdin=subprocess.PIPE)

    # Start server/client session and make the connection
    def start(self):
        # Start the MATLAB server in a new process
        print("Starting %s on ZMQ socket %s" % (self._program_name(), self.socket_addr))
        print("Send 'exit' command to kill the server")
        self._run_server()

        # Start the client
        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.REQ)
        self.socket.connect(self.socket_addr)

        self.started = True

        # Test if connection is established
        if self.is_connected():
            print("%s started and connected!" % self._program_name())
            return True
        else:
            print("%s failed to start" % self._program_name())
            return False

    def _response(self, **kwargs):
        req = json.dumps(kwargs, cls=PymatEncoder)
        self.socket.send_string(req)
        resp = self.socket.recv_string()
        return resp

    # Stop the Matlab server
    def stop(self):
        # Matlab should respond with "exit" if successful
        if self._response(cmd='exit') == "exit":
            print("%s closed" % self._program_name())

        self.started = False
        return True

    # To test if the client can talk to the server
    def is_connected(self):
        if not self.started:
            time.sleep(2)
            return False

        req = json.dumps(dict(cmd="connect"), cls=PymatEncoder)
        self.socket.send_string(req)

        start_time = time.time()
        while True:
            try:
                resp = self.socket.recv_string(flags=zmq.NOBLOCK)
                return resp == "connected"
            except zmq.ZMQError:
                sys.stdout.write('.')
                time.sleep(1)
                if time.time() - start_time > self.maxtime:
                    print("%s session timed out after %d seconds" % (self._program_name(), self.maxtime))
                    return False

    def is_function_processor_working(self):
        result = self.run_func('%s/usrprog/test_sum.m' % MATLAB_FOLDER,
                {'echo': '%s: Function processor is working!' % self._program_name()})
        return result['success'] == 'true'

    def _json_response(self, **kwargs):
        if self.running:
            time.sleep(0.05)
        return json.loads(self._response(**kwargs), object_hook=decode_pymat)

    # Run a function in Matlab and return the result
    def run_func(self, func_path, func_args=None):
        return self._json_response(cmd='run_function',
                                   func_path=func_path,
                                   func_args=func_args)

    # Run some code in Matlab command line provide by a string
    def run_code(self, code):
        return self._json_response(cmd='run_code', code=code)

    def get_variable(self, varname, default=None):
        response = self._json_response(cmd='get_var', varname=varname)
        if response['exists']:
            return response['var']
        else:
            return default

    def set_variable(self, varname, value):
        if isinstance(value, spmatrix):
            return self._set_sparse_variable(varname, value)
        return self.run_func('pymat_set_variable.m',
                             {'name': varname, 'value': value})

    def _set_sparse_variable(self, varname, value):
        value = value.todok()
        prefix = 'pymatbridge_temp_sparse_%s_' % uuid4().hex
        self.set_variable(prefix + 'keys', value.keys())
        # correct for 1-indexing in MATLAB
        self.run_code('{0}keys = {0}keys + 1;'.format(prefix))
        self.set_variable(prefix + 'values', value.values())
        cmd = "{1} = sparse({0}keys(:, 1), {0}keys(:, 2), {0}values');"
        result = self.run_code(cmd.format(prefix, varname))
        self.run_code('clear {0}keys {0}values'.format(prefix))
        return result


class Matlab(_Session):
    def __init__(self, executable='matlab', socket_addr=None,
                 id='python-matlab-bridge', log=False, maxtime=60,
                 platform=None, startup_options=None):
        """
        Initialize this thing.

        Parameters
        ----------

        executable : str
            A string that would start Matlab at the terminal. Per default, this
            is set to 'matlab', so that you can alias in your bash setup

        socket_addr : str
            A string that represents a valid ZMQ socket address, such as
            "ipc:///tmp/pymatbridge", "tcp://127.0.0.1:55555", etc.

        id : str
            An identifier for this instance of the pymatbridge.

        log : bool
            Whether to save a log file in some known location.

        maxtime : float
           The maximal time to wait for a response from matlab (optional,
           Default is 10 sec)

        platform : string
           The OS of the machine on which this is running. Per default this
           will be taken from sys.platform.

        startup_options : string
           Command line options to pass to MATLAB. Optional; sensible defaults
           are used if this is not provided.
        """
        if platform is None:
            platform = sys.platform
        if startup_options is None:
            if platform == 'win32':
                startup_options = ' -automation -noFigureWindows'
            else:
                startup_options = ' -nodesktop -nodisplay'
        if log:
            startup_options += ' -logfile ./pymatbridge/logs/matlablog_%s.txt' % id
        super(Matlab, self).__init__(executable, socket_addr, id, log, maxtime,
                                     platform, startup_options)

    def _program_name(self):
        return 'MATLAB'

    def _execute_flag(self):
        return '-r'


class Octave(_Session):
    def __init__(self, executable='octave', socket_addr=None,
                 id='python-matlab-bridge', log=False, maxtime=60,
                 platform=None, startup_options=None):
        """
        Initialize this thing.

        Parameters
        ----------

        executable : str
            A string that would start Octave at the terminal. Per default, this
            is set to 'octave', so that you can alias in your bash setup

        socket_addr : str
            A string that represents a valid ZMQ socket address, such as
            "ipc:///tmp/pymatbridge", "tcp://127.0.0.1:55555", etc.

        id : str
            An identifier for this instance of the pymatbridge.

        log : bool
            Whether to save a log file in some known location.

        maxtime : float
           The maximal time to wait for a response from octave (optional,
           Default is 10 sec)

        platform : string
           The OS of the machine on which this is running. Per default this
           will be taken from sys.platform.

        startup_options : string
           Command line options to pass to Octave. Optional; sensible defaults
           are used if this is not provided.
        """
        if startup_options is None:
            startup_options = '--silent --no-gui'
        super(Octave, self).__init__(executable, socket_addr, id, log, maxtime,
                                     platform, startup_options)

    def _program_name(self):
        return 'Octave'

    def _preamble_code(self):
        code = super(Octave, self)._preamble_code()
        if self.log:
            code.append("diary('./pymatbridge/logs/octavelog_%s.txt')" % self.id)
        return code

    def _execute_flag(self):
        return '--eval'



"""

matlab_magic
============

Magic command interface for interactive work with Matlab(R) via the pymatbridge


Note
====
Thanks to Max Jaderberg for his work on pymatbridge.

"""

import sys, os
import tempfile
from glob import glob
from shutil import rmtree
from getopt import getopt

import numpy as np
try:
    import scipy.io as sio
    has_io = True
except ImportError:
    has_io = False
    no_io_str = "Must have scipy.io to perform i/o"
    no_io_str += "operations with the Matlab session"

import IPython

ipython_version = int(IPython.__version__[0])

from IPython.core.displaypub import publish_display_data
from IPython.core.magic import (Magics, magics_class, cell_magic, line_magic,
                                line_cell_magic, needs_local_scope)
from IPython.testing.skipdoctest import skip_doctest
from IPython.core.magic_arguments import (argument, magic_arguments,
                                          parse_argstring)
from IPython.utils.py3compat import str_to_unicode, unicode_to_str, PY3

import pymatbridge as pymat
from .compat import text_type


class MatlabInterperterError(RuntimeError):
    """
    Some error occurs while matlab is running
    """
    def __init__(self, line, err):
        self.line = line
        self.err = err

    def __unicode__(self):
        s = "Failed to parse and evaluate line %r.\n Matlab error message: %r"%\
                (self.line, self.err)
        return s

    if PY3:
        __str__ = __unicode__
    else:
        def __str__(self):
            return unicode_to_str(text_type(self), 'utf-8')



@magics_class
class MatlabMagics(Magics):
    """
    A set of magics for interactive work with Matlab(R).
    """
    def __init__(self, shell,
                 matlab='matlab',
                 maxtime=60,
                 pyconverter=np.asarray,
                 cache_display_data=False):
        """
        Parameters
        ----------

        shell : IPython shell

        matlab : str
            The system call to start a matlab session. Allows you to choose a
            particular version of matlab if you want

        maxtime : float
           The maximal time to wait for responses for matlab (in seconds).
           Default: 10 seconds.

        pyconverter : callable
            To be called on matlab variables returning into the ipython
            namespace

        cache_display_data : bool
            If True, the published results of the final call to R are
            cached in the variable 'display_cache'.

        """
        super(MatlabMagics, self).__init__(shell)
        self.cache_display_data = cache_display_data

        self.Matlab = pymat.Matlab(matlab, maxtime=maxtime)
        self.Matlab.start()
        self.pyconverter = pyconverter

    def __del__(self):
        """shut down the Matlab server when the object dies.

        2FIX: this seems to not be called when ipython terminates. bleah.
        """
        try:
            self.Matlab.stop()
        except:
            raise

    def eval(self, line):
        """
        Parse and evaluate a single line of matlab
        """
        run_dict = self.Matlab.run_code(line)

        if run_dict['success'] == 'false':
            raise MatlabInterperterError(line, run_dict['content']['stdout'])

        # This is the matlab stdout:
        return run_dict

    def set_matlab_var(self, name, value):
        """
        Set up a variable in Matlab workspace
        """
        run_dict = self.Matlab.set_variable(name, value)

        if run_dict['success'] == 'false':
            raise MatlabInterperterError(line, run_dict['content']['stdout'])


    @magic_arguments()
    @argument(
        '-i', '--input', action='append',
        help='Names of input variable from shell.user_ns to be assigned to Matlab variables of the same names after calling self.pyconverter. Multiple names can be passed separated only by commas with no whitespace.'
        )

    @argument(
        '-o', '--output', action='append',
        help='Names of variables to be pushed from matlab to shell.user_ns after executing cell body and applying self.Matlab.get_variable(). Multiple names can be passed separated only by commas with no whitespace.'
        )

    @argument(
        '-s', '--silent', action='store_true',
        help='Do not display text output of MATLAB command'
        )

    @argument(
        'code',
        nargs='*',
        )


    @needs_local_scope
    @line_cell_magic
    def matlab(self, line, cell=None, local_ns=None):
        """

        Execute code in matlab

        """
        args = parse_argstring(self.matlab, line)

        # arguments 'code' in line are prepended to
        # the cell lines

        if cell is None:
            code = ''
            return_output = True
            line_mode = True
        else:
            code = cell
            return_output = False
            line_mode = False

        code = ' '.join(args.code) + code

        if local_ns is None:
            local_ns = {}


        if args.input:
            if has_io:
                for input in ','.join(args.input).split(','):
                    try:
                        val = local_ns[input]
                    except KeyError:
                        val = self.shell.user_ns[input]
                    # The _Session.set_variable function which this calls
                    # should correctly detect numpy arrays and serialize them
                    # as json correctly.
                    self.set_matlab_var(input, val)

            else:
                raise RuntimeError(no_io_str)

        text_output = ''
        #imgfiles = []

        try:
            if line_mode:
                e_s = "There was an error running the code:\n %s"%line
                result_dict = self.eval(line)
            else:
                e_s = "There was an error running the code:\n %s"%code
                result_dict = self.eval(code)
        except:
            e_s += "\n-----------------------"
            e_s += "\nAre you sure Matlab is started?"
            raise RuntimeError(e_s)



        text_output += result_dict['content']['stdout']
        # Figures get saved by matlab in reverse order...
        imgfiles = result_dict['content']['figures'][::-1]
        data_dir = result_dict['content']['datadir']

        display_data = []
        if text_output and not args.silent:
            if ipython_version < 3:
                display_data.append(('MatlabMagic.matlab',
                                     {'text/plain':text_output}))
            else:
                display_data.append({'text/plain':text_output})

        for imgf in imgfiles:
            if len(imgf):
                # Store the path to the directory so that you can delete it
                # later on:
                image = open(imgf, 'rb').read()
                if ipython_version < 3:
                    display_data.append(('MatlabMagic.matlab',
                                         {'image/png':image}))
                else:
                    display_data.append({'image/png': image})

        for disp_d in display_data:
            if ipython_version < 3:
                publish_display_data(disp_d[0], disp_d[1])
            else:
                publish_display_data(disp_d)

        # Delete the temporary data files created by matlab:
        if len(data_dir):
            rmtree(data_dir)

        if args.output:
            if has_io:
                for output in ','.join(args.output).split(','):
                    self.shell.push({output:self.Matlab.get_variable(output)})
            else:
                raise RuntimeError(no_io_str)


_loaded = False
def load_ipython_extension(ip, **kwargs):
    """Load the extension in IPython."""
    global _loaded
    if not _loaded:
        ip.register_magics(MatlabMagics(ip, **kwargs))
        _loaded = True

def unload_ipython_extension(ip):
    global _loaded
    if _loaded:
        magic = ip.magics_manager.registry.pop('MatlabMagics')
        magic.Matlab.stop()
        _loaded = False




"""pymatbridge version/release information"""

# Format expected by setup.py and doc/source/conf.py: string of form "X.Y.Z"
_version_major = 0
_version_minor = 4
_version_micro = 3#''  # use '' for first of series, number for 1 and above
    #_version_extra = dev'
_version_extra = ''  # Uncomment this for full releases

# Construct full version string from these.
_ver = [_version_major, _version_minor]
if _version_micro:
    _ver.append(_version_micro)
if _version_extra:
    _ver.append(_version_extra)

__version__ = '.'.join(map(str, _ver))

CLASSIFIERS = ["Development Status :: 3 - Alpha",
               "Environment :: Console",
               "Intended Audience :: Science/Research",
               "License :: OSI Approved :: BSD License",
               "Operating System :: OS Independent",
               "Programming Language :: Python",
               "Topic :: Scientific/Engineering"]

description = "pymatbridge is a set of python and matlab functions to allow these two systems to talk to each other"

long_description = """

Pymatbridge
===========

A python interface to call out to Matlab(R).


License information
===================

Copyright (c) 2012 -- , Max Jaderberg, Ariel Rokem, Haoxing Zhao
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

Neither the name of the Oxford University, Stanford University nor the names of
its contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""

NAME = "pymatbridge"
MAINTAINER = "Ariel Rokem"
MAINTAINER_EMAIL = "arokem@gmail.com"
DESCRIPTION = description
LONG_DESCRIPTION = long_description
URL = "https://github.com/arokem/python-matlab-bridge"
DOWNLOAD_URL = "https://github.com/arokem/python-matlab-bridge/archive/master.tar.gz"
LICENSE = "BSD"
AUTHOR = "https://github.com/arokem/python-matlab-bridge/contributors"
AUTHOR_EMAIL = "arokem@gmail.com"
PLATFORMS = "OS Independent"
MAJOR = _version_major
MINOR = _version_minor
MICRO = _version_micro
VERSION = __version__
PACKAGES = ['pymatbridge']
PACKAGE_DATA = {"pymatbridge": ["matlab/matlabserver.m", "matlab/messenger.*",
                                "matlab/usrprog/*", "matlab/util/*.m",
                                "matlab/util/json_v0.2.2/LICENSE",
                                "matlab/util/json_v0.2.2/README.md",
                                "matlab/util/json_v0.2.2/test/*",
                                "matlab/util/json_v0.2.2/json/*.m",
                                "matlab/util/json_v0.2.2/json/java/*",
                                "tests/*.py", "examples/*.ipynb"]}

REQUIRES = ['pyzmq']
#EXTRAS_REQUIRE = ['numpy', 'scipy', 'ipython']

BIN=['scripts/publish-notebook']



import numpy.testing as npt
import pymatbridge.publish as publish

def test_format_line():
    """
    Test that lines get formatted properly

    """
    npt.assert_equal(publish.format_line('%% This begins a new cell'),
                      (True, True, ' This begins a new cell\n'))

    npt.assert_equal(publish.format_line('% This should be just markdown'),
                      (False, True, ' This should be just markdown\n'))

    npt.assert_equal(publish.format_line('This is just code'),
                      (False, False, 'This is just code'))


def test_lines_to_notebook():
    """
    Test that conversion of some lines gives you a proper notebook
    """

    lines  = ["%% This is a first line\n",
              "\n",
              "% This should be in another cell\n",
              "a = 1; % This is code\n",
              "b = 2; % This is also code\n",
              "c = 3; % code in another cell\n"]

    nb = publish.lines_to_notebook(lines)

    npt.assert_equal(nb['worksheets'][0]['cells'][1]['source'][0],
                     ' This is a first line\n\n')



import pymatbridge as pymat
from pymatbridge.compat import unichr
import numpy.testing as npt
import test_utils as tu

class TestJson:

    # Start a Matlab session before doing any tests
    @classmethod
    def setup_class(cls):
        cls.mlab = tu.connect_to_matlab()

    # Tear down the Matlab session after all the tests are done
    @classmethod
    def teardown_class(cls):
        tu.stop_matlab(cls.mlab)


    # Add 1 to the argument and return it
    def test_demo_func(self):
        for i in range(5):
            res = self.mlab.run_func('demo_func.m', {'a': i})['result']
            ans = i + 1
            npt.assert_equal(res, ans, err_msg = "demo_func.m test failed")

    # Print some strange characters in Matlab, get them back and compare.
    def test_special_character(self):

        # Test 1:"hi"\n
        res = self.mlab.run_code('fprintf(1, char([34, 104, 105, 34, 10]));')['content']['stdout']
        ans = unichr(34) + unichr(104) + unichr(105) + unichr(34) + unichr(10)
        npt.assert_equal(res, ans, err_msg = "Special Character Test 1 failed")

        # Test 2:\b\n
        res = self.mlab.run_code('fprintf(1, char([8,10]));')['content']['stdout']
        ans = unichr(8) + unichr(10)
        npt.assert_equal(res, ans, err_msg = "Special Character Test 2 failed")

        # Test 3:\f\n
        res = self.mlab.run_code('fprintf(1, char([12,10]));')['content']['stdout']
        ans = unichr(12) + unichr(10)
        npt.assert_equal(res, ans, err_msg = "Special Character Test 3 failed")

        # Test 4:\r\n
        res = self.mlab.run_code('fprintf(1, char([13,10]));')['content']['stdout']
        ans = unichr(13) + unichr(10)
        npt.assert_equal(res, ans, err_msg = "Special Character Test 4 failed")

        # Test 5:\t\n
        res = self.mlab.run_code('fprintf(1, char([9,10]));')['content']['stdout']
        ans = unichr(9) + unichr(10)
        npt.assert_equal(res, ans, err_msg = "Special Character Test 5 failed")

        # Test 6:\\\n
        res = self.mlab.run_code('fprintf(1, char([92,92,10]));')['content']['stdout']
        ans = unichr(92) + unichr(10)   # Python already encoded double-slash as 92
        npt.assert_equal(res, ans, err_msg = "Special Character Test 6 failed")

        # Test 7:/\n
        res = self.mlab.run_code('fprintf(1, char([47,10]));')['content']['stdout']
        ans = unichr(47) + unichr(10)
        npt.assert_equal(res, ans, err_msg = "Special Character Test 7 failed")

        # Test 8: Lots of strange characters
        res = self.mlab.run_code('fprintf(1,char([47,92,92,47,8,12,10,13,9,12,8,8,12,47,34,47,10,10,47,34]));')['content']['stdout']
        ans = unichr(47) + unichr(92) + unichr(47) + unichr(8) + unichr(12) + unichr(10) \
                + unichr(13) + unichr(9) + unichr(12) + unichr(8) + unichr(8) + unichr(12) \
                + unichr(47) + unichr(34) + unichr(47) + unichr(10) + unichr(10) + unichr(47) + unichr(34)
        npt.assert_equal(res, ans, err_msg = "Special Character Test 8 failed")



import pymatbridge as pymat
from pymatbridge.compat import text_type
import numpy.testing as npt
import test_utils as tu


class TestRunCode:

    # Start a Matlab session before running any tests
    @classmethod
    def setup_class(cls):
        cls.mlab = tu.connect_to_matlab()

    # Tear down the Matlab session after running all the tests
    @classmethod
    def teardown_class(cls):
        tu.stop_matlab(cls.mlab)

    # Running 'disp()' in Matlab command window
    def test_disp(self):
        result1 = self.mlab.run_code("disp('Hello world')")['content']['stdout']
        result2 = self.mlab.run_code("disp('   ')")['content']['stdout']
        result3 = self.mlab.run_code("disp('')")['content']['stdout']

        npt.assert_equal(result1, "Hello world\n")
        npt.assert_equal(result2, "   \n")
        if tu.on_octave():
            npt.assert_equal(result3, '\n')
        else:
            npt.assert_equal(result3, "")

    # Make some assignments and run basic operations
    def test_basic_operation(self):
        result_assignment_a = self.mlab.run_code("a = 21.23452261")['content']['stdout']
        result_assignment_b = self.mlab.run_code("b = 347.745")['content']['stdout']
        result_sum = self.mlab.run_code("a + b")['content']['stdout']
        result_diff = self.mlab.run_code("a - b")['content']['stdout']
        result_product = self.mlab.run_code("a * b")['content']['stdout']
        result_division = self.mlab.run_code("c = a / b")['content']['stdout']

        if tu.on_octave():
            npt.assert_equal(result_assignment_a, text_type("a =  21.235\n"))
            npt.assert_equal(result_assignment_b, text_type("b =  347.75\n"))
            npt.assert_equal(result_sum, text_type("ans =  368.98\n"))
            npt.assert_equal(result_diff, text_type("ans = -326.51\n"))
            npt.assert_equal(result_product, text_type("ans =  7384.2\n"))
            npt.assert_equal(result_division, text_type("c =  0.061063\n"))
        else:
            npt.assert_equal(result_assignment_a, text_type("\na =\n\n   21.2345\n\n"))
            npt.assert_equal(result_assignment_b, text_type("\nb =\n\n  347.7450\n\n"))
            npt.assert_equal(result_sum, text_type("\nans =\n\n  368.9795\n\n"))
            npt.assert_equal(result_diff, text_type("\nans =\n\n -326.5105\n\n"))
            npt.assert_equal(result_product, text_type("\nans =\n\n   7.3842e+03\n\n"))
            npt.assert_equal(result_division, text_type("\nc =\n\n    0.0611\n\n"))

    # Put in some undefined code
    def test_undefined_code(self):
        success = self.mlab.run_code("this_is_nonsense")['success']
        message = self.mlab.run_code("this_is_nonsense")['content']['stdout']

        npt.assert_equal(success, "false")
        if tu.on_octave():
            npt.assert_equal(message, "'this_is_nonsense' undefined near line 1 column 1")
        else:
            npt.assert_equal(message, "Undefined function or variable 'this_is_nonsense'.")



import pymatbridge as pymat
import random as rd
import numpy as np
import numpy.testing as npt
import test_utils as tu

class TestArray:

    # Start a Matlab session before any tests
    @classmethod
    def setup_class(cls):
        cls.mlab = tu.connect_to_matlab()

    # Tear down the Matlab session after all the tests are done
    @classmethod
    def teardown_class(cls):
        tu.stop_matlab(cls.mlab)


    # Pass a 1000*1000 array to Matlab
    def test_array_size(self):
        array = np.random.random_sample((50,50))
        res = self.mlab.run_func("array_size.m",{'val':array})['result']
        npt.assert_almost_equal(res, array, decimal=8, err_msg = "test_array_size: error")





#!/usr/bin/env python
"""Setup file for python-matlab-bridge"""

import os
import sys
import shutil

# BEFORE importing distutils, remove MANIFEST. distutils doesn't properly
# update it when the contents of directories change.
if os.path.exists('MANIFEST'):
    os.remove('MANIFEST')

from distutils.core import setup

# Find the messenger binary file and copy it to /matlab folder.

def copy_bin(bin_path):
    if os.path.exists(bin_path):
        shutil.copy(bin_path, "./pymatbridge/matlab")
        return True
    else:
        return False

if sys.platform == "win32":
    raise ValueError("pymatbridge does not work on win32")
else:
    for copy_this in ["./messenger/mexmaci64/messenger.mexmaci64",
                      "./messenger/mexa64/messenger.mexa64",
                      "./messenger/mexw64/messenger.mexw64"]:
        copy_bin(copy_this)
        
# Get version and release info, which is all stored in pymatbridge/version.py
ver_file = os.path.join('pymatbridge', 'version.py')
exec(open(ver_file).read())

opts = dict(name=NAME,
            maintainer=MAINTAINER,
            maintainer_email=MAINTAINER_EMAIL,
            description=DESCRIPTION,
            long_description=LONG_DESCRIPTION,
            url=URL,
            download_url=DOWNLOAD_URL,
            license=LICENSE,
            classifiers=CLASSIFIERS,
            author=AUTHOR,
            author_email=AUTHOR_EMAIL,
            platforms=PLATFORMS,
            version=VERSION,
            packages=PACKAGES,
            package_data=PACKAGE_DATA,
            requires=REQUIRES,
            #extras_require=EXTRAS_REQUIRE,
            scripts=BIN
            )


# Now call the actual setup function
if __name__ == '__main__':
    setup(**opts)



import IPython.nbformat.current as nbformat
import numpy as np

def format_line(line):
    """
    Format a line of Matlab into either a markdown line or a code line.

    Parameters
    ----------
    line : str
        The line of code to be formatted. Formatting occurs according to the
        following rules:

        - If the line starts with (at least) two %% signs, a new cell will be
          started.

        - If the line doesn't start with a '%' sign, it is assumed to be legit
          matlab code. We will continue to add to the same cell until reaching
          the next comment line
    """
    if line.startswith('%%'):
        md = True
        new_cell = True
        source = line.split('%%')[1] + '\n'  # line-breaks in md require a line
                                             # gap!

    elif line.startswith('%'):
        md = True
        new_cell = False
        source = line.split('%')[1] + '\n'

    else:
        md = False
        new_cell = False
        source = line


    return new_cell, md, source


def mfile_to_lines(mfile):
    """
    Read the lines from an mfile

    Parameters
    ----------
    mfile : string
        Full path to an m file
    """
    # We should only be able to read this file:
    f = file(mfile, 'r')
    lines = f.readlines()
    f.close()
    return lines

def lines_to_notebook(lines, name=None):
    """
    Convert the lines of an m file into an IPython notebook

    Parameters
    ----------
    lines : list
        A list of strings. Each element is a line in the m file

    Returns
    -------
    notebook : an IPython NotebookNode class instance, containing the
    information required to create a file


    """
    source = []
    md = np.empty(len(lines), dtype=object)
    new_cell = np.empty(len(lines), dtype=object)
    for idx, l in enumerate(lines):
        new_cell[idx], md[idx], this_source = format_line(l)
        # Transitions between markdown and code and vice-versa merit a new
        # cell, even if no newline, or "%%" is found. Make sure not to do this
        # check for the very first line!
        if idx>1 and not new_cell[idx]:
            if md[idx] != md[idx-1]:
                new_cell[idx] = True

        source.append(this_source)
    # This defines the breaking points between cells:
    new_cell_idx = np.hstack([np.where(new_cell)[0], -1])

    # Listify the sources:
    cell_source = [source[new_cell_idx[i]:new_cell_idx[i+1]]
                   for i in range(len(new_cell_idx)-1)]
    cell_md = [md[new_cell_idx[i]] for i in range(len(new_cell_idx)-1)]
    cells = []

    # Append the notebook with loading matlab magic extension
    notebook_head = "import pymatbridge as pymat\n" + "ip = get_ipython()\n" \
                    + "pymat.load_ipython_extension(ip)"
    cells.append(nbformat.new_code_cell(notebook_head, language='python'))

    for cell_idx, cell_s in enumerate(cell_source):
        if cell_md[cell_idx]:
            cells.append(nbformat.new_text_cell('markdown', cell_s))
        else:
            cell_s.insert(0, '%%matlab\n')
            cells.append(nbformat.new_code_cell(cell_s, language='matlab'))

    ws = nbformat.new_worksheet(cells=cells)
    notebook = nbformat.new_notebook(metadata=nbformat.new_metadata(),
                                 worksheets=[ws])
    return notebook


def convert_mfile(mfile, outfile=None):
    """
    Convert a Matlab m-file into a Matlab notebook in ipynb format

    Parameters
    ----------
    mfile : string
        Full path to a matlab m file to convert

    outfile : string (optional)
        Full path to the output ipynb file

    """
    f = file(mfile, 'r')
    lines = f.readlines()
    f.close()
    nb = lines_to_notebook(lines)
    if outfile is None:
        outfile = mfile.split('.m')[0] + '.ipynb'
    nbfile = file(outfile, 'w')
    nbformat.write(nb, nbfile, format='ipynb')
    nbfile.close()

